# jhbuild - a build script for GNOME 1.x and 2.x
# Copyright (C) 2001-2004  James Henstridge
#
#   autobuild.py: build logic for a non-interactive reporting build
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import time
import subprocess
import locale

from jhbuild.utils import cmds
import buildscript

import xmlrpclib
from cStringIO import StringIO

def get_distro():
    # try using the lsb_release tool to get the distro info
    try:
        distro = cmds.get_output(['lsb_release', '--short', '--id']).strip()
        release = cmds.get_output(['lsb_release', '--short', '--release']).strip()
        codename = cmds.get_output(['lsb_release', '--short', '--codename']).strip()
        if codename:
            return '%s %s (%s)' % (distro, release, codename)
        else:
            return '%s %s' % (distro, release)
    except (OSError, IOError):
        pass

    # otherwise, look for a /etc/*-release file
    release_files = ['/etc/redhat-release', '/etc/debian_version' ]
    release_files.extend([ os.path.join('/etc', fname)
                           for fname in os.listdir('/etc')
                             if fname.endswith('release') \
                                 and fname != 'lsb-release' ])
    for filename in release_files:
        if os.path.exists(filename):
            return open(filename, 'r').readline().strip()

    # else:
    return None

def escape(string):
    return string.replace('&','&amp;').replace('<','&lt;').replace('>','&gt;')

def fix_encoding(string):
    charset = locale.getpreferredencoding()
    s = 'VERY BORKEN ENCODING'
    for encoding in [charset, 'utf-8', 'iso-8859-15']:
        try:
            s = unicode(string, encoding)
        except:
            continue
        break
    return s.encode('us-ascii', 'xmlcharrefreplace')

class AutobuildBuildScript(buildscript.BuildScript):
    xmlrpc_report_url = None
    verbose = False

    def __init__(self, config, module_list):
        buildscript.BuildScript.__init__(self, config, module_list)
        self.xmlrpc_report_url = config.autobuild_report_url
        self.verbose = config.verbose
        self.server = None
        self.modulefp = None
        self.phasefp = None
        os.environ['TERM'] = 'dumb'
        os.environ['LANG'] = 'C'
        for k in os.environ.keys():
            if k.startswith('LC_'):
                os.environ[k] = 'C'

    def timestamp(self):
        tm = time.time()
        s = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(tm))
        msecs = max(int((tm - int(tm)) * 1000), 0)
        return '%s.%03d' % (s, msecs)

    def message(self, msg, module_num=-1):
        '''Display a message to the user'''
        if self.phasefp:
            fp = self.phasefp
        elif self.modulefp:
            fp = self.modulefp
        else:
            fp = None

        if fp:
            fp.write(msg + '\n')

    def set_action(self, action, module, module_num=-1, action_target=None):
        if module_num == -1:
            module_num = self.module_num
        if not action_target:
            action_target = module.name

    def execute(self, command, hint=None):
        '''executes a command, and returns the error code'''

        if isinstance(command, (str, unicode)):
            self.phasefp.write('<span class="command">%s</span>\n'
                                % escape(command))
        else:
            self.phasefp.write('<span class="command">%s</span>\n'
                                % escape(' '.join(command)))

        if hint == 'cvs':
            def format_line(line, error_output, fp=self.phasefp):
                if line[-1] == '\n': line = line[:-1]
                if line.startswith('C '):
                    fp.write('<span class="conflict">%s</span>\n'
                                        % escape(line))
                else:
                    fp.write('%s\n' % escape(line))
            stderr = subprocess.STDOUT
        else:
            def format_line(line, error_output, fp=self.phasefp):
                if line[-1] == '\n': line = line[:-1]
                if error_output:
                    fp.write('<span class="error">%s</span>\n'
                                        % escape(line))
                else:
                    fp.write('%s\n' % escape(line))
            stderr = subprocess.PIPE

        p = subprocess.Popen(command, shell=isinstance(command, (str,unicode)),
                             close_fds=True,
                             stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                             stderr=stderr)
        cmds.pprint_output(p, format_line)
        return p.returncode

    def start_build(self):
        self.server = xmlrpclib.ServerProxy(self.xmlrpc_report_url, allow_none = True)

        # close stdin
        os.close(0)

        info = {}
        import socket
        un = os.uname()

        info['build_host'] = socket.gethostname()
        info['architecture'] = (un[0], un[2], un[4])

        distro = get_distro()
        if distro:
            info['distribution'] = distro

        info['module_set'] = self.config.moduleset

        self.build_id = self.server.start_build(info)
        
        if self.verbose:
            s = 'Starting Build #%s' % self.build_id
            print s
            print '=' * len(s)
            print ''


    def end_build(self, failures):
        self.server.end_build(self.build_id, failures)

    def start_module(self, module):
        if self.verbose:
            print 'Starting module', module
        self.server.start_module(self.build_id, module)
        self.current_module = module
        self.modulefp = StringIO()
        

    def end_module(self, module, failed):
        log = fix_encoding(self.modulefp.getvalue())
        self.modulefp = None
        self.server.end_module(self.build_id, module, log, failed)

    def start_phase(self, module, state):
        if self.verbose:
            print '  %s/phase: %s' % (module, state)
        self.server.start_phase(self.build_id, module, state)
        self.phasefp = StringIO()

    def end_phase(self, module, state, error):
        log = fix_encoding(self.phasefp.getvalue())
        self.phasefp = None
        self.server.end_phase(self.build_id, module, state, log, error)

    def handle_error(self, module, state, nextstate, error, altstates):
        '''handle error during build'''
        if self.verbose:
            print 'failure'
            log = None
            if self.phasefp:
                log = fix_encoding(self.phasefp.getvalue())
            elif self.modulefp:
                log = fix_encoding(self.modulefp.getvalue())
            if log:
                print log
        #self.message('error during stage %s of %s: %s' % (state, module.name,
        #                                                  error))
        return 'fail'


BUILD_SCRIPT = AutobuildBuildScript
